from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User
from django.shortcuts import render, redirect
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse
import os
from .models import Profile, mlModel, Billing, Record
from .forms import UserLoginForm, UserUploadForm, UploadModelForm, GenerateBillingsForm, UpdateUser


# Registration view
def register_view(request):
    if request.method == 'POST':
        form = UserCreationForm(request.POST)
        if form.is_valid():
            user = form.save()
            # Create a profile for the user
            role = request.POST.get('role', 'user')  # Default to 'user' if not provided
            Profile.objects.create(user=user, role=role)
            return redirect('login')
    else:
        form = UserCreationForm()
    return render(request, 'register.html', {'form': form})

# Login view
def login_view(request):
    if request.method == 'POST':
        form = UserLoginForm(request.POST)
        if form.is_valid():
            username = form.cleaned_data['username']
            password = form.cleaned_data['password']
            user = authenticate(request, username=username, password=password)
            if user is not None:
                login(request, user)
                # Get the profile and role
                profile = user.profile
                role = profile.role
                # Redirect to the correct profile page based on the user's role
                return redirect('profile')
            else:
                # Handle invalid login
                return render(request, 'login.html', {'error': 'Invalid credentials'})
    else:
        form = UserLoginForm()
    return render(request, 'login.html', {'form': form})

def profile(request):
    return render(request, 'profile.html')

def userUpload(request):
    form = UserUploadForm(initial={'username': request.user.username}) # sets username in form.
    if request.method == 'POST':
        form = UserUploadForm(request.POST or None, request.FILES or None)
        if form.is_valid():
            form.save()
        return redirect('profile')
    context = {'form': form}
    return render(request, 'modelForms/userUpload.html', context)

def mlUpload(request):
    form = UploadModelForm(initial={'username': request.user.username})
    if request.method == 'POST':
        form = UploadModelForm(request.POST or None, request.FILES or None)
        if form.is_valid():
            form.save()
        return redirect('profile')
    context = {'form': form}
    return render(request, 'modelForms/mlUpload.html', context)

def genBillings(request):
    form = GenerateBillingsForm(initial={'username': request.user.username})
    if request.method == 'POST':
        form = GenerateBillingsForm(request.POST)
        if form.is_valid():
            form.save()
        return redirect('profile')
    context = {'form': form}
    return render(request, 'modelForms/genBillings.html', context)

def manageUsers(request):
    profile = Profile.objects.all()
    return render(request, 'modelForms/manageUsers.html', {'profile': profile})

def updateUsers(request, id):
    user = Profile.objects.get(id=id)
    form = UpdateUser(instance=user)

    if request.method == 'POST':
        form=UpdateUser(request.POST, instance=user)
        if form.is_valid():
            form.save()
            return redirect('manageUsers')

    context = {'form': form}
    return render(request, 'modelForms/updateUsers.html', context)

def interactionView(request):
    interactions = Record.objects.all().values()
    context = {}
    context = {'interactions': interactions}
    return render(request, 'modelForms/interactionView.html', context)

##### DELETION SECTIONS

def deleteRecord(request, id):
    record = Record.objects.get(id=id)
    if request.method == 'POST':
        record.delete()
        # figure out how to delete file in records directory at same time.
        return redirect('interactionView')
    context = {'record': record}
    return render(request, 'modelForms/deleteRecord.html', context)


def deleteUsers(request, id):
    user = Profile.objects.get(user_id=id)
    user2 = User.objects.get(id=id)
    if request.method == 'POST':
        user.delete()
        user2.delete()
        # because of how users are stored, we need to delete from both Profile and User
        return redirect('manageUsers')

    context = {'user': user}
    return render(request, 'modelForms/deleteUsers.html', context)

##### LOGOUT LOGIN STUFF

def logout_view(request):
    logout(request)
    return redirect('home')  # Replace 'home' with the URL name for your homepage

# Profile view (only accessible if logged in)
@login_required
def profile_view(request):
    return render(request, 'profile.html', {'user': request.user})
